package info.textgrid.middleware;

import java.io.IOException;
import java.text.ParseException;
import java.util.Hashtable;
import java.util.Map;
import java.util.logging.Logger;
import org.elasticsearch.action.search.SearchRequest;
import org.elasticsearch.action.search.SearchResponse;
import org.elasticsearch.action.search.SearchScrollRequest;
import org.elasticsearch.client.RequestOptions;
import org.elasticsearch.common.unit.TimeValue;
import org.elasticsearch.index.query.BoolQueryBuilder;
import org.elasticsearch.index.query.QueryBuilders;
import org.elasticsearch.index.query.RangeQueryBuilder;
import org.elasticsearch.search.SearchHit;
import org.elasticsearch.search.builder.SearchSourceBuilder;
import org.json.JSONObject;
import info.textgrid.middleware.oaipmh.GetRecordType;
import info.textgrid.middleware.oaipmh.ListRecordsType;
import info.textgrid.middleware.oaipmh.RecordType;
import info.textgrid.middleware.oaipmh.ResumptionTokenType;

/**
 * @author Maximilian Brodhun, SUB Göttingen
 * @author Stefan E. Funk, SUB Göttingen
 * @version 2022-09-19
 * @since
 */
public class RecordListDelivererIDIOM extends RecordListDelivererAbstract {

  // **
  // STATICS
  // **

  private static Logger log = Logger.getLogger(RecordListDelivererIDIOM.class.getName());
  private static Map<String, Integer> cursorCollector = new Hashtable<String, Integer>();

  // **
  // CLASS
  // **

  private RecordDelivererIDIOM idiomRecord;
  // Set default to 30, can be changed in oaipmh.properties.
  private int idiomResponseSize = 30;

  /**
   * @param textgrid
   * @param dariah
   */
  public RecordListDelivererIDIOM(boolean textgrid, boolean dariah) {
    super(textgrid, dariah);
  }

  /**
   *
   */
  @Override
  public ListRecordsType getRecords(final String from, final String to, final String set,
      final String resumptionToken) throws ParseException, IOException {

    ListRecordsType recordList = new ListRecordsType();

    RangeQueryBuilder rangeQuery = QueryBuilders.rangeQuery("lastModified").from(from).to(to);

    BoolQueryBuilder projectQuery = QueryBuilders.boolQuery().must(
        QueryBuilders.matchPhraseQuery("project.id", "TGPR-0e926f53-1aba-d415-ecf6-539edcd8a318"));

    BoolQueryBuilder artefactQuery = QueryBuilders.boolQuery()
        .must(QueryBuilders.matchPhraseQuery("format", "text/tg.inputform+rdf+xml"))
        .must(QueryBuilders.matchPhraseQuery("notes", "ARTEFACT"));

    BoolQueryBuilder conedakorQuery = QueryBuilders.boolQuery()
        .must(QueryBuilders.matchPhraseQuery("format", "application/json"))
        .must(QueryBuilders.matchPhraseQuery("notes", "ConedaKorMediumData"));

    BoolQueryBuilder both = QueryBuilders.boolQuery().should(artefactQuery).should(conedakorQuery);

    BoolQueryBuilder queryBuilder =
        QueryBuilders.boolQuery().must(rangeQuery).must(projectQuery.filter(both));
    BoolQueryBuilder recordFilterForClassicMayan = queryBuilder;
    // QueryBuilders.boolQuery().must(rangeQuery).should(recordFilteroClassicMayanARTEFACT).should(recordFilterForClassicMayanCONEDAKOR);

    SearchResponse scrollResp;

    SearchRequest searchRequest = new SearchRequest("textgrid-nonpublic");
    SearchSourceBuilder searchSourceBuilder = new SearchSourceBuilder();

    scrollResp = null;

    searchSourceBuilder.query(recordFilterForClassicMayan);
    searchSourceBuilder.size(this.idiomResponseSize);
    searchRequest.source(searchSourceBuilder);

    if (resumptionToken != null) {
      SearchScrollRequest scrollRequest = new SearchScrollRequest(resumptionToken);
      scrollRequest.scroll(TimeValue.timeValueHours(24L));
      try {
        scrollResp = this.oaiEsClient.getEsClient().scroll(scrollRequest, RequestOptions.DEFAULT);
      } catch (IOException e) {
        // TODO Auto-generated catch block
        e.printStackTrace();
      }
    } else {
      searchRequest.source(searchSourceBuilder);
      searchRequest.scroll(TimeValue.timeValueHours(24L));
      try {
        scrollResp = this.oaiEsClient.getEsClient().search(searchRequest, RequestOptions.DEFAULT);
      } catch (IOException e) {
        // TODO Auto-generated catch block
        e.printStackTrace();
      }
    }

    String scrollID = scrollResp.getScrollId();

    long completeListSize = scrollResp.getHits().totalHits;

    log.fine("total hits: " + completeListSize);

    if (completeListSize > 0) {
      setFoundItems(true);
      int i = 0;

      for (SearchHit hit : scrollResp.getHits().getHits()) {
        i++;

        log.fine("hit no." + i + ": id=" + hit.getId());

        String textgridURI =
            OAIPMHUtilities.fieldLoader(new JSONObject(hit.getSourceAsMap()), "textgridUri");

        log.fine("textgridURI: " + textgridURI);

        // Create new record here! We must not set only references here!
        RecordType record = new RecordType();
        RecordType sourceRecord =
            this.idiomRecord.getRecordById(textgridURI.replace(".0", "")).getRecord();
        record.setHeader(sourceRecord.getHeader());
        record.setMetadata(sourceRecord.getMetadata());

        recordList.getRecord().add(record);

        int size = recordList.getRecord().size();

        log.finest("RECORD LIST CONTENT (" + size + "):");
        for (int j = 0; j < size; j++) {
          log.finest(
              "\t[" + j + "] --> " + recordList.getRecord().get(j).getHeader().getIdentifier());
        }
      }

      // Check the need for a resumption token!
      ResumptionTokenType responseToken = OAIPMHUtilities.getResumptionToken(completeListSize,
          resumptionToken, cursorCollector, scrollID, this.idiomResponseSize, i);
      if (responseToken != null) {
        recordList.setResumptionToken(responseToken);
      }
    } else {
      setFoundItems(false);
    }

    log.fine("record list size: " + recordList.getRecord().size());

    return recordList;
  }

  // **
  // GETTERS & SETTERS
  // **

  /**
   * @return
   */
  public int getIdiomResponseSize() {
    return this.idiomResponseSize;
  }

  /**
   * @param idiomResponseSize
   */
  public void setIdiomResponseSize(int idiomResponseSize) {
    this.idiomResponseSize = idiomResponseSize;
  }

  /**
   * @return
   */
  public RecordDelivererIDIOM getIdiomRecord() {
    return this.idiomRecord;
  }

  /**
   * @param idiomRecord
   */
  public void setIdiomRecord(RecordDelivererIDIOM idiomRecord) {
    this.idiomRecord = idiomRecord;
  }

  /**
   * @return
   */
  public static Map<String, Integer> getCursorCollector() {
    return cursorCollector;
  }

}
